//	Dimitrios Milios 932

package genetics;

import java.util.*;

public class GeneticAlgorithm {

	CrossOver xOverOperator;
	FitnessEvaluation fitnessFuntion;
	SolutionFactory factory;
	Selection select;
	Population pop;
	Random rand;
	
	public GeneticAlgorithm(CrossOver xOverOperator, FitnessEvaluation fitnessFuntion, SolutionFactory factory, Selection select)
	{
		super();
		this.xOverOperator = xOverOperator;
		this.fitnessFuntion = fitnessFuntion;
		this.factory = factory;
		this.select = select;
		rand = new Random( System.currentTimeMillis() );
	}

	public void execute( int popNumber, int repetitions )
	{
		xOverOperator.setChromosomeLenght( factory.getChromosomeLenght() );
		pop = new Population( popNumber, fitnessFuntion, factory );
		select.setPopulation( pop );
		for( int i = 0; i < repetitions; i++ )
			procceedOneGeneration();
		System.out.println();
		System.out.println( "Avg Value: " + pop.getAvgValue() );
		System.out.print( "Best Value: " );
		if ( select.isMaxBetter() )
		{
			System.out.print( pop.getMaxIndividual().getValue() );
			pop.getMaxIndividual().print();
		}
		else
		{
			System.out.print( pop.getMinIndividual().getValue() );
			pop.getMinIndividual().print();
		}
	}
	
	private void procceedOneGeneration()
	{
		select.preparations();
		while( pop.getTotalNum() > pop.getCurrentNum() )
		{
			Individual [] parents = select.findParents();
			Individual [] children = xOverOperator.produceChildren(parents[0], parents[1]);
			for ( int i = 0; i < children.length; i++ )
				pop.add( children[i] );
		}
		pop.evaluateAll();
	}
	
}
